﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Ogc.Filter.ComparisonOps
{
    public abstract class BinaryComparisonOp : ComparisonOp
    {
        IExpression expra;
        IExpression exprb;
        bool matchCase;
        public bool MatchCase
        {
            get { return matchCase; }
            set { matchCase = value; }
        }

        protected BinaryComparisonOp(IExpression expra,IExpression exprb)
        {
            this.expra = expra;
            this.exprb = exprb;
        }

        public override bool Test(IPropertyGetter getter)
        {
            return Evaluate(expra.Evaluate(getter), exprb.Evaluate(getter));
        }

        protected abstract bool Evaluate(object a, object b);
    }

    public class PropertyIsEqualTo : BinaryComparisonOp
    {
        public PropertyIsEqualTo(IExpression expra, IExpression exprb)
            : base(expra, exprb)
        {
        }

        protected override bool Evaluate(object a, object b)
        {
            if (a is string && b is string)
                return a.ToString().Equals(b.ToString(), MatchCase ? StringComparison.Ordinal : StringComparison.OrdinalIgnoreCase);
            return a.Equals(b);
        }
    }

    public class PropertyIsNotEqualTo : BinaryComparisonOp
    {
        public PropertyIsNotEqualTo(IExpression expra, IExpression exprb)
            : base(expra, exprb)
        {
        }

        protected override bool Evaluate(object a, object b)
        {
            if (a is string && b is string)
                return !a.ToString().Equals(b.ToString(), MatchCase ? StringComparison.Ordinal : StringComparison.OrdinalIgnoreCase);
            return !a.Equals(b);
        }
    }

    public class PropertyIsLessThan : BinaryComparisonOp
    {
        public PropertyIsLessThan(IExpression expra, IExpression exprb)
            : base(expra, exprb)
        {
        }

        protected override bool Evaluate(object a, object b)
        {
            IComparable c = a as IComparable;
            if (c != null)
                return c.CompareTo(b) < 0;
            return false;
        }
    }

    public class PropertyIsGreaterThan : BinaryComparisonOp
    {
        public PropertyIsGreaterThan(IExpression expra, IExpression exprb)
            : base(expra, exprb)
        {
        }

        protected override bool Evaluate(object a, object b)
        {
            IComparable c = a as IComparable;
            if (c != null)
                return c.CompareTo(b) > 0;
            return false;
        }
    }

    public class PropertyIsLessThanOrEqualTo : BinaryComparisonOp
    {
        public PropertyIsLessThanOrEqualTo(IExpression expra, IExpression exprb)
            : base(expra, exprb)
        {
        }

        protected override bool Evaluate(object a, object b)
        {
            IComparable c = a as IComparable;
            if (c != null)
                return c.CompareTo(b) <= 0;
            return false;
        }
    }

    public class PropertyIsGreaterThanOrEqualTo : BinaryComparisonOp
    {
        public PropertyIsGreaterThanOrEqualTo(IExpression expra, IExpression exprb)
            : base(expra, exprb)
        {
        }

        protected override bool Evaluate(object a, object b)
        {
            IComparable c = a as IComparable;
            if (c != null)
                return c.CompareTo(b) >= 0;
            return false;
        }
    }
}
