﻿using System;
using System.Collections.Generic;
using System.Text;
using SharpMap.Geometries;

namespace Ogc.Filter.SpatialOps
{
    public abstract class BinarySpatialOpType : SpatialOp
    {
        Geometry geometry;
        public Geometry Geometry
        {
            get { return geometry; }
            set { geometry = value; }
        }

        BoundingBox envalop;
        public BoundingBox Envalop
        {
            get { return envalop; }
            set { envalop = value; }
        }

        protected BinarySpatialOpType(string geoPropertyName, Geometry geometry)
            : base(geoPropertyName)
        {
            this.geometry = geometry;
        }

        protected BinarySpatialOpType(string geoPropertyName, BoundingBox envalop)
            : base(geoPropertyName)
        {
            this.envalop = envalop;
        }

        protected abstract bool Evaluate(Geometry a, Geometry b);
        protected abstract bool Evaluate(Geometry a, BoundingBox b);

        protected override bool Test(IGeomeryPropertyGetter getter)
        {
            Geometry geoa = getter.GetGeometryProperty(GeoPropertyName);
            if (geometry != null)
                return Evaluate(geoa, geometry);
            else if (envalop != null)
                return Evaluate(geoa, envalop);
            return true;
        }
    }

    public class Equals : BinarySpatialOpType
    {
        public Equals(string geoPropertyName, BoundingBox envalop)
            : base(geoPropertyName, envalop)
        {
        }

        public Equals(string geoPropertyName, Geometry geometry)
            : base(geoPropertyName, geometry)
        {
        }

        protected override bool Evaluate(Geometry a, Geometry b)
        {
            return a.Equals(b);
        }

        protected override bool Evaluate(Geometry a, BoundingBox b)
        {
            return a.GetBoundingBox().Equals(b);
        }
    }

    public class Disjoint : BinarySpatialOpType
    {
        //public Disjoint(string geoPropertyName, BoundingBox envalop)
        //    : base(geoPropertyName, envalop)
        //{
        //}

        public Disjoint(string geoPropertyName, Geometry geometry)
            : base(geoPropertyName, geometry)
        {
        }

        protected override bool Evaluate(Geometry a, Geometry b)
        {
            return a.Disjoint(b);
        }

        protected override bool Evaluate(Geometry a, BoundingBox b)
        {
            throw new System.NotSupportedException();
        }
    }

    public class Touches : BinarySpatialOpType
    {
        public Touches(string geoPropertyName, BoundingBox envalop)
            : base(geoPropertyName, envalop)
        {
        }

        public Touches(string geoPropertyName, Geometry geometry)
            : base(geoPropertyName, geometry)
        {
        }

        protected override bool Evaluate(Geometry a, Geometry b)
        {
            return a.Touches(b);
        }

        protected override bool Evaluate(Geometry a, BoundingBox b)
        {
            return a.GetBoundingBox().Touches(b);
        }
    }

    public class Within : BinarySpatialOpType
    {
        //public Within(string geoPropertyName, BoundingBox envalop)
        //    : base(geoPropertyName, envalop)
        //{
        //}

        public Within(string geoPropertyName, Geometry geometry)
            : base(geoPropertyName, geometry)
        {
        }

        protected override bool Evaluate(Geometry a, Geometry b)
        {
            return a.Within(b);
        }

        protected override bool Evaluate(Geometry a, BoundingBox b)
        {
            throw new System.NotSupportedException();
        }
    }

    public class Overlaps : BinarySpatialOpType
    {
        //public Overlaps(string geoPropertyName, BoundingBox envalop)
        //    : base(geoPropertyName, envalop)
        //{
        //}

        public Overlaps(string geoPropertyName, Geometry geometry)
            : base(geoPropertyName, geometry)
        {
        }

        protected override bool Evaluate(Geometry a, Geometry b)
        {
            return a.Overlaps(b);
        }

        protected override bool Evaluate(Geometry a, BoundingBox b)
        {
            throw new System.NotSupportedException();
        }
    }

    public class Crosses : BinarySpatialOpType
    {
        //public Crosses(string geoPropertyName, BoundingBox envalop)
        //    : base(geoPropertyName, envalop)
        //{
        //}

        public Crosses(string geoPropertyName, Geometry geometry)
            : base(geoPropertyName, geometry)
        {
        }

        protected override bool Evaluate(Geometry a, Geometry b)
        {
            return a.Crosses(b);
        }

        protected override bool Evaluate(Geometry a, BoundingBox b)
        {
            throw new System.NotSupportedException();
        }
    }

    public class Intersects : BinarySpatialOpType
    {
        //public Intersects(string geoPropertyName, BoundingBox envalop)
        //    : base(geoPropertyName, envalop)
        //{
        //}

        public Intersects(string geoPropertyName, Geometry geometry)
            : base(geoPropertyName, geometry)
        {
        }

        protected override bool Evaluate(Geometry a, Geometry b)
        {
            return a.Intersects(b);
        }

        protected override bool Evaluate(Geometry a, BoundingBox b)
        {
            return a.GetBoundingBox().Intersects(b);
            //throw new System.NotSupportedException();
        }
    }

    public class Contains : BinarySpatialOpType
    {
        public Contains(string geoPropertyName, BoundingBox envalop)
            : base(geoPropertyName, envalop)
        {
        }

        public Contains(string geoPropertyName, Geometry geometry)
            : base(geoPropertyName, geometry)
        {
        }

        protected override bool Evaluate(Geometry a, Geometry b)
        {
            return a.Contains(b);
        }

        protected override bool Evaluate(Geometry a, BoundingBox b)
        {
            return a.GetBoundingBox().Contains(b);
            //throw new System.NotSupportedException();
        }
    }
}
