﻿using System;
using System.Collections.Generic;
using System.Text;
using SharpMap.Layers;
using SharpMap.Data.Providers;
using Ogc.Sld;
using SharpMap.Geometries;
using SharpMap.CoordinateSystems.Transformations;
using SharpMap.Data;
using SharpMap.Styles;

namespace Ogc.SharpMapExtentions
{
    public class StyledLayer:Layer
    {
        Rule[] rules;
        public Rule[] Rules
        {
            get { return rules; }
            set { rules = value; }
        }
        
        IProvider dataSource;
        public SharpMap.Data.Providers.IProvider DataSource
        {
            get { return dataSource; }
            set { dataSource = value; }
        }

        ICoordinateTransformation coordinateTransform;
        public SharpMap.CoordinateSystems.Transformations.ICoordinateTransformation CoordinateTransform
        {
            get { return coordinateTransform; }
            set { coordinateTransform = value; }
        }
        public override void Render(System.Drawing.Graphics g, SharpMap.Map map)
        {
 			BoundingBox envelope = map.Envelope; 
			if (coordinateTransform != null)
				envelope = SharpMap.CoordinateSystems.Transformations.GeometryTransform
                    .TransformBox(envelope, coordinateTransform.MathTransform.Inverse());

            FeatureDataSet ds = new FeatureDataSet();
            this.dataSource.Open();
            this.dataSource.ExecuteIntersectionQuery(envelope, ds);
            this.dataSource.Close();

            foreach (Rule rule in rules)
            {
                foreach (FeatureDataTable dt in ds.Tables)
                {
                    for (int i = 0; i < dt.Count; ++i)
                    {
                        FeatureDataRow dr = dt[i];
                        Symbolizer symbolizer;
                        if (rule.Test(dr, map.Zoom, out symbolizer))
                            symbolizer.Render(g, map, dr);
                    }
                }
            }
        }

        public override SharpMap.Geometries.BoundingBox Envelope
        {
            get { return dataSource.GetExtents(); }
        }

        public override object Clone()
        {
            throw new Exception("The method or operation is not implemented.");
        }
    }

}
