﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using SharpMap.Data.Providers;
using SharpMap.Geometries;
using Ogc.Wfs;

namespace Ogc.SharpMapExtentions
{
    public class WfsProvider : IProvider
    {
        string uri;
        string typeName;
        string[] propertieNames;
        string geoPropertyName;
        string srsName;
        BoundingBox extents;

        private RequestGetFeatureBoxParameters BuildRequestParameters(SharpMap.Geometries.BoundingBox bbox)
        {
            RequestGetFeatureBoxParameters p = new RequestGetFeatureBoxParameters();
            p.Uri = uri;
            TypeNameParameter typeNameP = new TypeNameParameter();
            typeNameP.TypaName = typeName;
            typeNameP.PropertyNames.AddRange(propertieNames);
            BBoxFilterParameter filteP = new BBoxFilterParameter();
            filteP.GeoPropertyName = geoPropertyName;
            EnvelopeParameter envelopP = new EnvelopeParameter();
            envelopP.SrsName = srsName;
            envelopP.MinX = bbox.Min.X;
            envelopP.MinY = bbox.Min.Y;
            envelopP.MaxX = bbox.Max.X;
            envelopP.MaxY = bbox.Max.Y;
            filteP.Envelope = envelopP;
            typeNameP.Filter = filteP;
            p.TypeNames.Add(typeNameP);
            return p;
        }

        public WfsProvider(string uri, string typeName, string[] propertieNames, string geoPropertyName, string srsName, BoundingBox extents)
        {
            this.uri = uri;
            this.typeName = typeName;
            this.propertieNames = propertieNames;
            this.geoPropertyName = geoPropertyName;
            this.srsName = srsName;
            this.extents = extents;
        }

        #region IProvider 成员

        public System.Collections.Generic.List<SharpMap.Geometries.Geometry> GetGeometriesInView(SharpMap.Geometries.BoundingBox bbox)
        {
            return new GmlGeometryBuilder(new Client().GetFeature(BuildRequestParameters(bbox)))
                .BuildGeometrys(geoPropertyName);
        }

        public System.Collections.Generic.List<uint> GetObjectIDsInView(SharpMap.Geometries.BoundingBox bbox)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public SharpMap.Geometries.Geometry GetGeometryByID(uint oid)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ExecuteIntersectionQuery(SharpMap.Geometries.Geometry geom, SharpMap.Data.FeatureDataSet ds)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void ExecuteIntersectionQuery(SharpMap.Geometries.BoundingBox box, SharpMap.Data.FeatureDataSet ds)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public int GetFeatureCount()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public SharpMap.Data.FeatureDataRow GetFeature(uint RowID)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public SharpMap.Geometries.BoundingBox GetExtents()
        {
            return extents;
        }

        public string ConnectionID
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        public void Open()
        {

        }

        public void Close()
        {

        }

        public bool IsOpen
        {
            get { return true; }
        }

        public int SRID
        {
            get
            {
                return -1;
            }
            set
            {
                throw new Exception("The method or operation is not implemented.");
            }
        }

        #endregion

        #region IDisposable 成员

        public void Dispose()
        {

        }

        #endregion
    }

    class Helper
    {
        [Obsolete("构建Namesapce的算法不够强壮，只能应付目前的数据")]
        public static XmlNamespaceManager BuildNamespaceManager(XmlDocument doc)
        {
            XmlNamespaceManager mgr = new XmlNamespaceManager(doc.NameTable);

            foreach (XmlAttribute i in doc.DocumentElement.Attributes)
            {
                if (i.Name.StartsWith("xmlns:"))
                {
                    int idx = i.Name.IndexOf(":") + 1;
                    mgr.AddNamespace(i.Name.Substring(idx, i.Name.Length - idx), i.Value);
                }
            }
            return mgr;
        }
    }
}
